#INCLUDE "INKEY.CH"

/*
This program provides a color pallatte so that a user can select colors within
within a program.  The first bit of code and the SetUpColors() function is just
for demonstration purposes. You can use it, modify it or trash it.  The guts of
this code is in the COLORS() function.

This example uses a database called COLORS, and is indexed on the field
C_USERID.  A user id is passed to the SetUpColors() function.  The function
seeks the color record for the user id and then invokes the COLORS()
function.  If no user id is found, a new record is added.  (Of course, you
will probably modify this to suit your system needs.)  Once finished selecting
all colors for various screen elements, the new colors settings are written
to the database.

For demo purposes, enter a userid at startup.

*/


/* assigns a default value to a variable if its NIL */
#command DEFAULT <p> TO <val> [,<pn> TO <valn>];
         => ;
         <p> = iif(<p> = NIL, <val>, <p>);
         [;<pn> = iif(<pn> = NIL, <valn>, <pn>)]


SET(36,24)
SETBLINK(.F.)
SETCOLOR("W+/B,W+/R,,,N/BG")
CLEAR
cUserID := SPACE(20)
DO WHILE LASTKEY() != K_ESC
   CLEAR
   @ 01,01 SAY "Enter a userid (for demo purposes): " GET cUserID PICT "@!"
   @ 02,01 SAY "[Esc] to quit."
   SETCURSOR(1)
   READ
   SETCURSOR(0)
   IF LASTKEY() != K_ESC
      SetUpColors(cUserID)
   ELSE
      EXIT
   ENDIF
ENDDO
SET COLOR TO
CLEAR
RETURN NIL

****************************************
FUNC SetupColors( UserID )
****************************************
LOCAL nOldArea := SELECT(), lFirstTime := .T., nChoice := 1
LOCAL nPromptRow := 03, nPromptCol := 04, A

USE COLORS INDEX COLORS NEW

SEEK UserID
IF .NOT. FOUND()
// RETURN NIL
   APPEND BLANK
ENDIF
PRIV aNewColors[ FCOUNT()-1 ]
PRIV aPrompts[ FCOUNT(), 2 ]
FOR A := 1 TO FCOUNT()-1
   aNewColors[A] := &(FIELDNAME(A))
NEXT

DEFAULT UserID to SPACE(40)

aPrompts[02,1] := " A Menu text         " ; aPrompts[02,2] := "Select color for the text of the menu."
aPrompts[01,1] := " B Menu selected     " ; aPrompts[01,2] := "Select color for the highlighted menu item."
aPrompts[03,1] := " C Menu title        " ; aPrompts[03,2] := "Select color for the title area of the menu."
aPrompts[04,1] := " D Dialog text       " ; aPrompts[04,2] := "Select color for the text that make up dialog (response) and text boxes."
aPrompts[05,1] := " E Dialog title      " ; aPrompts[05,2] := "Select color for the title area of the dialog box."
aPrompts[06,1] := " F Dialog selected   " ; aPrompts[06,2] := "Select color for the selected button in the dialog box."
aPrompts[07,1] := " G Dialog unselected " ; aPrompts[07,2] := "Select color for the unselected button(s) in the dialog box."
aPrompts[08,1] := " H Dialog hot key    " ; aPrompts[08,2] := "Select color for the hot key on each of the dialog box buttons."
aPrompts[09,1] := " I Help text         " ; aPrompts[09,2] := "Select color for the text of popup menu windows."
aPrompts[10,1] := " J Help title        " ; aPrompts[10,2] := "Select color for the title area of the help windows."
aPrompts[11,1] := " K Field selected    " ; aPrompts[11,2] := "Select color for the input field that currently has input focus."
aPrompts[12,1] := " L Field unselected  " ; aPrompts[12,2] := "Select color for the input field(s) that do not have input focus."
aPrompts[13,1] := " M Desktop           " ; aPrompts[13,2] := "Select color for the background/backdrop of the screen."
aPrompts[14,1] := " N Status line       " ; aPrompts[14,2] := "Select color for the status line."
aPrompts[15,1] := " O Error messages    " ; aPrompts[15,2] := "Select color for error messages."
aPrompts[16,1] := " X Exit              " ; aPrompts[16,2] := "Exit color selection."


SETCOLOR("W+/B,W+/R,,,N/BG")
CLEAR

// Display color pallatte only (initial display)
Colors( 03, 29, .F., lFirstTime, 240,,.T. )

lFirstTime := .F.

DO WHILE .T.
   FOR A = 1 TO LEN(aPrompts)
      @ (nPromptRow-1)+A,nPromptCol PROMPT aPrompts[A,1] MESSAGE PADC(aPrompts[A,2],MAXCOL()+1)
   NEXT
   MENU TO nChoice
   @ 24,00 SAY PADC("Navigation Keys:  Home, End, PgUp, PgDn, Arrow keys",80)
   IF nChoice = 0 .OR. nChoice = LEN(aPrompts)  // Remove .OR. if you dont have exit as last prompt.
      EXIT
   ENDIF
   // Insert code here for
   aNewColors[nChoice] := Colors( 03, 29, .F., lFirstTime, 240, aNewColors[nChoice] )
ENDDO

FOR A = 1 TO FCOUNT()-1  // Last field should be userid, hence -1.
   REPLACE &(FIELDNAME(A)) WITH aNewColors[A]
NEXT
REPLACE C_USERID WITH UserID

RETURN NIL

***************************************
STATIC FUNC COLORS( T, L, lSetBlink, lReColor, nCharString, cInitColor, lPallatteOnly )
***************************************
*   Ŀ
* ͵ COLORS()                  ͻ
*   ͻ
*  Written    06/92                                                          
*  Author     Robert Kratcoski                                               
*  Purpose    Provide a color palette and chose a color pair.                
*  Parameters T         =   Top row where color palette will begin.          
*                           DEFAULT: 4  (to center on screen)                
*             L         =   Left column where color palette will begin.      
*                           DEFAULT: 16 (to center on screen)                
*             lSetBlink =   Toggle between blinking foreground colors or     
*                           high intensity background colors.                
*                           DEFAULT: .T. (for blinking foreground colors)    
*             lReColor  =   Re-color the palatte area.  .T. = Draw color     
*                           palatte, .F. = do not draw color palatte.        
*                           DEFAULT: .T.                                     
*             lCharString = ASCII value of character to display for each     
*                           color pair.                                      
*                           DEFAULT:  254 ()                                
*             cInitColor =  Initial color pair to place color selection box  
*                           on.  Example: "W+*/B"                            
*             lPallateOnly= Display color pallate only.  Good for initial    
*                           display with menu before selecting first time.   
*  Returns    Color pair of the selected string.                             
*  Comments   It is a good idea to draw the palatte only the first time you  
*             invoke this function.  If a user is going to select color      
*             pairs for various screen attributes, it only needs to be drawn 
*             once before proceeding to the next attribute.  Redrawing the   
*             the palatte each and every time (especially on slow machines)  
*             is time consuming.                                             
* ͼ
#xtranslate MakeColor(<nF>, <nB>) => ;
if(<nB> <= 7,                        ;
   aColors\[<nF> + 1\] + "/" +       ;
   aColors\[<nB> + 1\],              ;
   aColors\[<nF> + 1\] + "*/" +      ;
   aColors\[<nB> - 8 + 1\])

LOCAL aColors    := { "N",    "B",     "G",     "BG", ;
                      "R",    "RB",    "GR",    "W",  ;
                      "N+",   "B+",    "G+",    "BG+",;
                      "R+",   "RB+",   "GR+",   "W+" }, ;
      nSetCursor := SETCURSOR(),;
      lBlinkSave := SETBLINK(),;
      cOldColor  := SETCOLOR(),;
      cSelArea, nKey, cFore, cBack, nFore, nBack, cString, cSelString

T              := IF( VALTYPE(T) == "N", T, 4)
L              := IF( VALTYPE(L) == "N", L, 16)
lSetBlink      := IF( VALTYPE(lSetBlink) == "L", lSetBlink, .T.)
lReColor       := IF( VALTYPE(lReColor) <> "L", .T., lReColor)
nCharString    := IF( VALTYPE(nCharString) == "N", nCharString, 254)
cString        := " " + chr(nCharString) + " "
cInitColor     := IF( VALTYPE(cInitColor)=="C",UPPER(cInitColor),"B/N")
lPallatteOnly  := IF( VALTYPE(lPallatteOnly)<>"L",.F.,lPallatteOnly)
cSelString     := chr(nCharString)

SetCursor(0)
lBlinkSave  = SETBLINK()
SETBLINK(lSetBlink)
DISPBEGIN()
IF lReColor
   FOR nFore := 0 to 15
      FOR nBack := 0 to 15
         SetColor(MakeColor(nFore, nBack))
         @ nFore+T, (nBack*Len(cString)+L) SAY  cString
      NEXT
   NEXT
ENDIF
DISPEND()

IF lPallatteOnly
   SETCOLOR(cOldColor)
   RETURN NIL
ENDIF

nKey := 0
cFore := SUBS( STRTRAN(cInitColor,"*",""), 1, AT("/",STRTRAN(cInitColor,"*",""))-1 )
cBack := SUBS( STRTRAN(cInitColor,"*",""), AT("/",STRTRAN(cInitColor,"*",""))+1)
cBack := IF("," $ cBack,SUBS(cBack,1,AT(",",cBack)-1),cBack)
nFore := ColorToNumber(ALLTRIM(cFore),aColors)
nBack := ColorToNumber(ALLTRIM(cBack),aColors)
nBack := IF("*" $ cInitColor, nBack+8, nBack)

DO WHILE nKey != K_ESC .AND. nKey != K_ENTER
   cSelArea = SaveScreen( (nFore-1+T),(nBack * Len(cString))+L,(nFore+T+1), (nBack * Len(cString))+L+2 )
   SetColor(MakeColor(15, nBack))
   @ (nFore-1+T),(nBack * Len(cString))+L,(nFore+T+1), (nBack * Len(cString))+L+2 BOX "Ŀ "
   SetColor(MakeColor(nFore, nBack))
   @ nFore+T, (nBack * Len(cString))+L+1 SAY cSelString
   DO WHILE .T.
      @nFore+T,(nBack * Len(cString))+L+1 SAY ""
      nKey := InKey(.1)
      IF nKey <> 0
         EXIT
      ENDIF
      @nFore+T,(nBack * Len(cString))+L+1 SAY "/"
      nKey := InKey(.1)
      IF nKey <> 0
         EXIT
      ENDIF
      @nFore+T,(nBack * Len(cString))+L+1 SAY ""
      nKey := InKey(.1)
      IF nKey <> 0
         EXIT
      ENDIF
      @nFore+T,(nBack * Len(cString))+L+1 SAY "\"
      nKey := InKey(.1)
      IF nKey <> 0
         EXIT
      ENDIF
      @nFore+T,(nBack * Len(cString))+L+1 SAY ""
      nKey := InKey(.1)
      IF nKey <> 0
         EXIT
      ENDIF
      @nFore+T,(nBack * Len(cString))+L+1 SAY "/"
      nKey := InKey(.1)
      IF nKey <> 0
         EXIT
      ENDIF
      @nFore+T,(nBack * Len(cString))+L+1 SAY ""
      nKey := InKey(.1)
      IF nKey <> 0
         EXIT
      ENDIF
      @nFore+T,(nBack * Len(cString))+L+1 SAY "\"
      nKey := InKey(.1)
      IF nKey <> 0
         EXIT
      ENDIF
   ENDDO
   RestScreen( (nFore-1+T),(nBack * Len(cString))+L,(nFore+T+1), (nBack * Len(cString))+L+2, cSelArea )
   @ nFore+T, (nBack * Len(cString))+L SAY cString
   DO CASE
   CASE nKey == K_UP
      nFore := Max(0, nFore - 1)
   CASE nKey == K_DOWN
      nFore := Min(15, nFore + 1)
   CASE nKey == K_LEFT
      nBack := Max(0, nBack - 1)
   CASE nKey == K_RIGHT
      nBack := Min(15, nBack + 1)
   CASE nKey == K_HOME
      nFore := 4; nBack := 3   // near upper left corner
*     nFore := 4; nBack := 4   // upper left corner
   CASE nKey == K_END
      nFore := 11; nBack := 3  // near lower left corner
*     nFore := 15; nBack := 0  // lower left corner
   CASE nKey == K_PGUP
      nFore := 4; nBack := 12  // near upper right corner
*     nFore := 0; nBack := 15  // upper right corner
   CASE nKey == K_PGDN
      nFore := 11; nBack := 12  // near lower right corner
*     nFore := 15; nBack := 15  // lower right corner
   ENDCASE
ENDDO
@ nFore+T, (nBack * Len(cString))+L SAY cString
SETCURSOR(nSetCursor)
SETBLINK(lBlinkSave)
SETCOLOR(cOldColor)
RETURN IF(nKey == K_ESC,"W/N",MakeColor(nFore, nBack))

*   Ŀ
* ͵ Update History ͻ
*   ͻ
*   d 04/93  n Robert Kratcoski c Added the nCharString, cInitColor,         
*                                 and lPallatteOnly parameters.              
*   d        n                  c                                            
* ͼ

****************************************
STATIC FUNC ColorToNumber( cColorString, aColors )
****************************************
// cColorString is not a color pair.  e.g. pass "W+" for bright white.

LOCAL A, B, nColorNumber := 0, lMatch
FOR A = 0 TO 15
   lMatch = .T.
   FOR B = 1 TO LEN(cColorString)
      lMatch := IF( ! SUBS(cColorString,B,1) $ aColors[A+1],.F.,lMatch )
   NEXT
   IF lMatch
      nColorNumber = A
      A := 16  // to exit
   ENDIF
NEXT
RETURN nColorNumber


